try:
    from ._version import version as __version__
except ImportError:
    __version__ = "unknown"
__author__ = "Alister Burt"
__email__ = "alisterburt@gmail.com"

from .emanjson2imodxf import emanjson2imodxf



from __future__ import annotations

import json
from typing import TYPE_CHECKING, Tuple

import numpy as np

if TYPE_CHECKING:
    from os import PathLike


def read_tlt_params(json_file: PathLike) -> Tuple[np.ndarray]:
    """Read an EMAN2 json file and extract alignment parameters.

    In EMAN2 tilt-series alignment translations are applied first, followed by
    rotations.

    Parameters
    ----------
    json_file : PathLike
        An EMAN2 json file containing alignment parameters from a tilt-series
        alignment experiment.

    Returns
    -------
    dxy, theta, projection_angles : tuple[np.ndarray]
        shifts and rotation angles from the json file. Shifts are an (n, 2, 1)
        array of dx, dy in pixels. theta is an (n, ) array of rotation angles
        in radians. ytilt is an (n, ) array of tilt angles for reconstruction.
    """
    with open(json_file) as f:
        data = json.load(f)
    tlt_params = np.array(data['tlt_params'])

    # extract useful information
    # reshape shifts to (n, 2, 1) array of column vectors
    dx = tlt_params[:, 0]
    dy = tlt_params[:, 1]
    dxy = np.stack((dx, dy), axis=-1).reshape((-1, 2, 1))
    theta = np.deg2rad(tlt_params[:, 2])

    projection_angles = tlt_params[:, 2]

    return dxy, theta, projection_angles


def angle2rotm(theta: np.ndarray) -> np.ndarray:
    """Convert angles into a stack of 2D rotation matrices.

    R(theta) = [[cos(theta), -sin(theta)],
                [sin(theta), cos(theta)]])

    Parameters
    ----------
    theta : np.ndarray
        An (n, ) array of angles in radians.

    Returns
    -------
    rotation_matrices : np.ndarray
        An (n, 2, 2) stack of rotation matrices.
    """
    cos_theta = np.cos(theta)
    sin_theta = np.sin(theta)
    r = np.array([[cos_theta, -sin_theta],
                  [sin_theta, cos_theta]])
    return r.swapaxes(0, 2)




from pathlib import Path

import numpy as np

from emanjson2imodxf import emanjson2imodxf

TEST_DIRECTORY = Path(__file__).parent
EMAN_JSON_FILE = TEST_DIRECTORY / 'example.json'
IMOD_XF_FILE = TEST_DIRECTORY / 'example.xf'
IMOD_TLT_FILE = TEST_DIRECTORY / 'example.tlt'


def test_emanjson2imodxf():
    """Check that output matches expected xf and tlt files.
    """
    xf, tilt_angles = emanjson2imodxf(EMAN_JSON_FILE)

    expected_xf = np.loadtxt(IMOD_XF_FILE)
    np.testing.assert_allclose(xf, expected_xf, atol=1e-3)

    expected_tlt = np.loadtxt(IMOD_TLT_FILE)
    np.testing.assert_allclose(tilt_angles, expected_tlt, atol=1e-2)


from os import PathLike

import click
import numpy as np

from .data_munging import read_tlt_params, angle2rotm


def emanjson2imodxf(input_json_file: PathLike):
    # Get useful info from EMAN json file
    # shifts in px, theta in radians, y tilt in degrees
    dxy_eman, theta, tilt_angles = read_tlt_params(json_file=input_json_file)
    rotation_matrices = angle2rotm(theta)

    # EMAN translates then rotates, IMOD rotates then translates
    # R(x+dx, y+dy) = R(x, y) + R(dx, dy)
    # IMOD (dx, dy) is the rotated EMAN (dx, dy)
    dxy_imod = rotation_matrices @ dxy_eman

    # get lines for IMOD xf file
    A11 = rotation_matrices[:, 0, 0]
    A12 = rotation_matrices[:, 0, 1]
    A21 = rotation_matrices[:, 1, 0]
    A22 = rotation_matrices[:, 1, 1]
    dx_imod = dxy_imod[:, 0].squeeze()
    dy_imod = dxy_imod[:, 1].squeeze()

    xf = np.stack((A11, A12, A21, A22, dx_imod, dy_imod), axis=-1)
    return xf, tilt_angles


@click.command()
@click.option(
    '-i', '--input', 'input_json_file',
    help='EMAN2 json file',
    type=click.Path(exists=True),
    required=True,
)
@click.option(
    '-o', '--output-basename', 'output_basename',
    help='basename for IMOD .xf and .tlt files',
    type=click.Path(),
    required=True,
)
def cli(input_json_file, output_basename):
    """Convert an EMAN2 json file into an IMOD xf file.
    """
    output_xf_file = f'{output_basename}.xf'
    output_tlt_file = f'{output_basename}.tlt'

    click.echo(f'Input file: {input_json_file}')
    click.echo(f'Output files: {output_xf_file}, {output_tlt_file}')

    xf, tilt_angles = emanjson2imodxf(input_json_file)
    np.savetxt(output_xf_file, xf, fmt='%.6f', delimiter=' ')
    np.savetxt(output_tlt_file, tilt_angles, fmt='%.2f')
    click.echo(f'Success!')
